package gregapi.tileentity.energy;

import static gregapi.data.CS.*;
import gregapi.data.LH;
import gregapi.data.LH.Chat;
import gregapi.data.TD;
import gregapi.tileentity.machines.ITileEntitySwitchableOnOff;
import gregapi.util.UT;

import java.util.List;

import net.minecraft.entity.Entity;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;

public abstract class TileEntityBase11Motor extends TileEntityBase10EnergyConverter implements ITileEntitySwitchableOnOff {
	protected boolean mCounterClockwise = F, mFast = F, oCounterClockwise = F, oFast = F;
	
    @Override
	public void readFromNBT2(NBTTagCompound aNBT) {
    	super.readFromNBT2(aNBT);
        if (aNBT.hasKey(NBT_STATE)) mCounterClockwise = aNBT.getBoolean(NBT_STATE);
        if (aNBT.hasKey(NBT_VISUAL)) mFast = aNBT.getBoolean(NBT_VISUAL);
    }
    
    @Override
	public void writeToNBT2(NBTTagCompound aNBT) {
    	super.writeToNBT2(aNBT);
        UT.NBT.setBoolean(aNBT, NBT_STATE, mCounterClockwise);
        UT.NBT.setBoolean(aNBT, NBT_VISUAL, mFast);
    }
    
	@Override
	public void addToolTips(List aList, ItemStack aStack, boolean aF3_H) {
		super.addToolTips(aList, aStack, aF3_H);
		aList.add(Chat.DGRAY	+ LH.get(LH.TOOL_TO_SET_DIRECTION_MONKEY_WRENCH));
	}
	
	@Override
	public long onToolClick2(String aTool, long aRemainingDurability, long aQuality, Entity aPlayer, List<String> aChatReturn, IInventory aPlayerInventory, boolean aSneaking, ItemStack aStack, byte aSide, float aHitX, float aHitY, float aHitZ) {
		long rReturn = super.onToolClick2(aTool, aRemainingDurability, aQuality, aPlayer, aChatReturn, aPlayerInventory, aSneaking, aStack, aSide, aHitX, aHitY, aHitZ);
		if (rReturn > 0) return rReturn;
		
		if (isClientSide()) return 0;
		if (aTool.equals(TOOL_monkeywrench)) {mCounterClockwise=!mCounterClockwise; return 10000;}
		
		return 0;
	}
	
    @Override
    public void doConversion(long aTimer) {
    	long tOutput = UT.Code.units(mEnergy, mInput, mOutput, F), tMaxOut = getEnergySizeOutputMax(mEnergyTypeEmitted, SIDE_ANY);
    	mActive = (tOutput >= getEnergySizeOutputMin(mEnergyTypeEmitted, SIDE_ANY));
    	mFast = (tOutput > mOutput);
		mEmitsEnergy = F;
    	
    	if (mActive) {
			if (tOutput > tMaxOut && TD.Energy.ALL_COMSUMPTION_LIMITED.contains(mEnergyTypeAccepted)) tOutput = tMaxOut;
    		if (tOutput > tMaxOut) {
    			overload(mEnergy, mEnergyTypeEmitted);
    		} else {
				if (TD.Energy.ALL_SIZE_IRRELEVANT.contains(mEnergyTypeEmitted)) {
					long tEmittedPackets = ITileEntityEnergy.Util.emitEnergyToNetwork(mEnergyTypeEmitted, 1, tOutput*mMultiplier, this);
					if (!mWasteEnergy) mEnergy -= UT.Code.units(tEmittedPackets, mOutput*mMultiplier, mInput, T);
					mEmitsEnergy = (tEmittedPackets > 0);
				} else {
					long tEmittedPackets = ITileEntityEnergy.Util.emitEnergyToNetwork(mEnergyTypeEmitted, mCounterClockwise?tOutput:-tOutput, mMultiplier, this);
					if (!mWasteEnergy) mEnergy -= UT.Code.units(tEmittedPackets * tOutput, mOutput*mMultiplier, mInput, T);
					mEmitsEnergy = (tEmittedPackets > 0);
				}
    		}
    	}
		if (mWasteEnergy) mEnergy = Math.max(0, mEnergy-getEnergySizeInputMax(mEnergyTypeAccepted, SIDE_ANY));
    }
    
    @Override
    public boolean onTickCheck(long aTimer) {
    	return mCounterClockwise != oCounterClockwise || mFast != oFast || super.onTickCheck(aTimer);
    }
    
	@Override
    public void onTickResetChecks(long aTimer, boolean aIsServerSide) {
    	super.onTickResetChecks(aTimer, aIsServerSide);
    	oCounterClockwise = mCounterClockwise;
		oFast = mFast;
    }
    
	@Override
	public byte getVisualData() {return (byte)(super.getVisualData() | (mCounterClockwise?2:0) | (mFast?4:0));}
	
	@Override
	public void setVisualData(byte aData) {
		super.setVisualData(aData);
		mCounterClockwise = ((aData & 2) != 0);
		mFast = ((aData & 4) != 0);
	}
}